/* -*-mode:C;tab-width:8-*-
 * powerd   Monitor the UPS status.
 *
 * Copyright (C) 1993        Miquel van Smoorenburg
 * Copyright (C) 1995        Alessandro Rubini
 * Copyright (C) 2001        Umberto Zappi
 *
 * This program is designed to work with SysVInit-2.4 for Linux and later, by
 * Miquel van Smoorenburg. It may work with other operating systems.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* define magic keyword for handshake in udp connections */
#define CURRENT_MAGIC "powerd-2.1"
#define MAGIC_2_0     "powerd-2.0"

#define P_CFG_FILE      PREFIX "/etc/powerd.conf" /* default config file */
#define MAX_SZPATH 	256	                  /* maximum size of pathname */


#ifdef linux
#  define P_PWRSTAT     "/etc/powerstatus"       /* default power status file */
/* info of mounted fs (as alternative of /etc/mtab or /etc/fstab) */ 
#  define P_MNT_INFO    "/proc/mounts"
#  define MOUNT_FS_RW   "/sbin/mount -w -n -o remount %s"
#  define MOUNT_FS_RO   "/sbin/mount -r -n -o remount %s"
/* #  define MOUNT_ALL    "/sbin/mount -avt nonfs" */
#else
#  error "Only Linux is supported, by now"
#endif

#define BIT_OUT 0x10000 /* to mark serial bits */

/* Boolean values */
#define FALSE   0
#define TRUE    !FALSE

/* Used as parameter 'mode' of the functions CheckSerial(), CheckSocket(),
 * CheckFIFO(), CheckChild()
 */
#define MODE_OPEN 0
#define MODE_READ 1

/*
 * DEBUG_LVLx depends on runtime options, and is there to help
 * understanding what is the program doing
 */

#define DEBUG_LVL1 (options[O_DEBUGLEVEL].iv >= 1) /* basic debug */
#define DEBUG_LVL2 (options[O_DEBUGLEVEL].iv >= 2) /* more verbose */
#define DEBUG_LVL3 (options[O_DEBUGLEVEL].iv >= 3) /* more verbose (loop) */
#define DEBUG_LVL9 (options[O_DEBUGLEVEL].iv >= 9) /* for code developers */
/* Debug info usefull for code developers */
#ifdef DEBUG
  #define PDEBUG_L9(args) do \
                            { if (DEBUG_LVL9) \
                                fprintf args; \
                            } \
                          while(0)
#else
  #define PDEBUG_L9(args)
#endif

/* Logging */
#define PERROR(s)         do \
                            { if (DEBUG_LVL1) \
                                fprintf(stderr,"%s: %s: %s\n", \
				    CurrTime(),(s),strerror(errno)); \
                              else \
                                syslog(LOG_ERR,"%s: %s: %s\n",prgname,(s), \
				   strerror(errno)); \
		            } \
                          while(0)

#define MESSAGE(s)        do \
                            { if (DEBUG_LVL1) \
                                fprintf(stderr,"%s: %s\n",CurrTime(),(s)); \
                              else \
                                syslog(LOG_ERR,"%s: %s\n",prgname,(s)); \
                            } \
                          while(0)


enum actions {ACT_ABORT=-2,      /* must be negative - used internally */
              ACT_NOTHING=-1,    /* must be negative - used internally */
	      ACT_OK='O',        /* as (O)k */   /* used for communications */
	      ACT_FAIL='F',      /* as (F)ail */ /* used for communications */
	      ACT_FAILLOW='L',   /* as (L)ow */  /* used for communications */
	      ACT_ASK='A'        /* as (A)sk */  /* used for communications */
	     };

/* actions used for communications in powerd 2.0 */
enum act_2_0 {ACT_FAIL_2_0=0, ACT_OK_2_0=1, ACT_ASK_2_0=254};

enum fsstatus {FS_RO, FS_RW};	/* read-only o read-write filesystem */


/*---- Defines structures -------------------------------------------*/
typedef struct Option {
  char *name;
  short multi_allowed;
  short usage;         /* to detect multiple lines (config file) */
  enum { OT_BOOL,
	 OT_STRING,
	 OT_FILE,
	 OT_RSNAME,
	 OT_STR_OR_LEV,
	 OT_LEVEL,
	 OT_INTEGER,
	 OT_END } type;
  int   iv;    /* integer */
  char *sv;    /* string  */
}    Option;

struct client {
  struct sockaddr_in addr;
  int addrlen;
  char magic[15];
  struct client *next;
  };

struct assert {
  int bit;
  int val;
  int sleep;
  struct assert *next;
};

struct powerd_message {
  unsigned char cmd; /* silly, isn't it? */
  char magic[15];
};
 
/*---- Prototypes ---------------------------------------------------*/
static int LoadOptions(int argc, char **argv, struct assert **p2Assertions);
static int ParseLine(char *opt, char *svalue, struct assert **p2Assertion);
static void PrintOptions(void);
static void PrintAssertions(struct assert *pAssert);
static int (*CheckUniqueTask(Option options[]))(int,int);
static void RedirectStderr(const char *logfile);
static int OpenUdpServer(int udp_port);
static int MainCheck(int sensefd, int serverfd, int current_status,
                     int (*handler)(int,int), Option options[]);
static unsigned char GetRunlevel(void);
static dev_t DeviceOf(const char *pathname);
static struct mntent *MntentOf(const char *pathname);
static char *MountPointOf(const char *pathname);
static int IsROFS(const char *pathname);
static int GetMountFS(const char *pathname);
static char *CurrTime(void);
static void CleanUsage(Option options[]);
static struct assert *AppendAssertList(struct assert **p2Assert);
#ifdef EXPERIMENTAL
static struct assert *GetLastAssert(struct assert *pAssert);
#endif
static void FreeAssertList(struct assert *pAssert);
static int CheckString(char *buffer);
static int CheckSerial(int mode, int arg);
static int CheckSocket(int mode, int arg);
static int CheckFIFO(int mode, int arg);
static int CheckChild(int mode, int arg);
static void SendPulse(int fd, struct assert *pAssert);
static int TranslateAction(int action, char *magic);
static int ManageClient(int fd, struct client **list, int current_status);
static struct client *InsertClientList(struct client **p2Client);
static void FreeClientList(struct client *pClient);
static int TellClients(int action, int serverfd, struct client *clPtr);
static int PowerFail(int action); /*  sys-dep */


/* End of file powerd.h */
